import {Component} from '@angular/core'
import {ErrorHandlingComponent} from '../../../shared/components/subscribing.component'
import {RecipeService} from '../../services/recipe.service'
import {EmployeePermission} from '../../../shared/model/employee'
import {AccountService} from '../../../accounts/services/account.service'
import {Recipe} from '../../../shared/model/recipe.model'
import {ActivatedRoute, Router} from '@angular/router'
import {ErrorModel, ErrorService} from '../../../shared/service/error.service'

@Component({
  selector: 'cre-list',
  templateUrl: './list.component.html',
  styleUrls: ['./list.component.sass']
})
export class ListComponent extends ErrorHandlingComponent {
  recipes$ = this.recipeService.allSortedByCompany
  tableCols = ['name', 'description', 'color', 'gloss', 'sample', 'iconNotApproved', 'buttonView', 'buttonEdit']
  searchQuery = ""
  panelForcedExpanded = false
  recipesHidden = []

  handledErrorModels: ErrorModel[]

  constructor(
    private recipeService: RecipeService,
    private accountService: AccountService,
    errorService: ErrorService,
    router: Router,
    activatedRoute: ActivatedRoute
  ) {
    super(errorService, activatedRoute, router)
  }

  ngOnInit() {
    super.ngOnInit();
  }

  searchRecipe(recipe: Recipe) {
    if (this.searchQuery.length > 0) {
      this.panelForcedExpanded = true
    }
    const positive = this.searchString(recipe.name) ||
      this.searchString(recipe.description) ||
      (recipe.sample && this.searchString(recipe.sample.toString()))
    this.recipesHidden[recipe.id] = !positive
    return positive
  }

  isCompanyHidden(companyRecipes: Recipe[]): boolean {
    return (this.searchQuery && this.searchQuery.length > 0) && companyRecipes.map(r => this.recipesHidden[r.id]).filter(r => !r).length <= 0
  }

  isLightColor(recipe: Recipe): boolean {
    // https://stackoverflow.com/questions/12043187/how-to-check-if-hex-color-is-too-black
    const c = recipe.color.substring(1)      // strip #
    const rgb = parseInt(c, 16)   // convert rrggbb to decimal
    const r = (rgb >> 16) & 0xff  // extract red
    const g = (rgb >> 8) & 0xff  // extract green
    const b = (rgb >> 0) & 0xff  // extract blue

    const luma = 0.2126 * r + 0.7152 * g + 0.0722 * b // per ITU-R BT.709
    return luma > 200
  }

  get hasEditPermission(): boolean {
    return this.accountService.hasPermission(EmployeePermission.EDIT_RECIPE)
  }

  private searchString(value: string): boolean {
    return value.toLowerCase().indexOf(this.searchQuery.toLowerCase()) >= 0
  }
}
